@@ -2,8 +2,14 @@
 
 #written by Jeremy M. Beaulieu
 
-OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA"), nboot=100, alpha, sigma.sq, theta, theta0, simmap.tree=FALSE, scaleHeight=FALSE, root.station=TRUE, clade=NULL, mserr="none", diagn=FALSE, quiet=TRUE, warn=FALSE){
+OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA","OUMVA"), nboot=100, alpha, sigma.sq, theta, theta0, simmap.tree=FALSE, root.age=NULL, scaleHeight=FALSE, root.station=TRUE, clade=NULL, mserr="none", diagn=FALSE, quiet=TRUE, warn=FALSE){
 	
+    if(is.null(root.age)){
+        if(any(branching.times(phy)<0)){
+            stop("Looks like your tree is producing negative branching times. Must input known root age of tree.", .call=FALSE)
+        }
+    }
+
 	#the matrix we are building to store the results:
 	res<-c()
 	#if alpha is NA set to a really small number -- this is only relevant for BM models:
@@ -15,7 +21,7 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 		tmp.phy<-phy
 		if(mserr=="known"){
 			if(!dim(data)[2]==4){
-				stop("You specified measurement error should be incorporated, but this information is missing")
+				stop("You specified measurement error should be incorporated, but this information is missing.", .call=FALSE)
 			}else{
 				#Now lengthen the terminal branches to reflect the intraspecific variation at the tips:
 				terminals <- tmp.phy$edge[,2] <= Ntip(tmp.phy)
@@ -25,10 +31,10 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
 		}
 		#This calls the OUwie simulator and simulates datasets:
         if(mserr == "none"){
-            tmp <- OUwie.sim(tmp.phy, data, simmap.tree=simmap.tree, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0, mserr=mserr)
+            tmp <- OUwie.sim(tmp.phy, data, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0, mserr=mserr)
         }
         if(mserr == "known"){
-            tmp <- OUwie.sim(tmp.phy, data[,c(1,2,4)], simmap.tree=simmap.tree, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0, mserr=mserr)
+            tmp <- OUwie.sim(tmp.phy, data[,c(1,2,4)], simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, alpha=alpha, sigma.sq=sigma.sq, theta=theta, theta0=theta0, mserr=mserr)
         }
 		#OUwie.sim outputs the trait file in the order of the tree, but the trait file is likely not to be this way. So I alphabetized the input trait file above, and I do the same to the simulated trait file:
 		data <- data[order(data[,1]),]
@@ -40,7 +46,7 @@ OUwie.boot <- function(phy, data, model=c("BM1","BMS","OU1","OUM","OUMV","OUMA",
             data[,3] <- tmp[,3]
         }
 		#Now run OUwie, using the measurement error if it is contained within the data, to estimate the parameters from the simulated data:
-		tmp <- OUwie(phy, data, model=model, simmap.tree=simmap.tree, scaleHeight=scaleHeight, root.station=root.station, clade=clade, mserr=mserr, diagn=diagn, quiet=quiet, warn=warn)
+		tmp <- OUwie(phy, data, model=model, simmap.tree=simmap.tree, root.age=root.age, scaleHeight=scaleHeight, root.station=root.station, clade=clade, mserr=mserr, diagn=diagn, quiet=quiet, warn=warn)
 		#Now bind all the relevant output together
 		if(model == "BM1" | model == "BMS" | model == "OU1"){
 			res <- rbind(res, c(tmp$solution[1,], tmp$solution[2,], tmp$theta[1,1]))
