@@ -2,7 +2,7 @@
 
 #written by Jeremy M. Beaulieu
 
-varcov.ou <- function(phy, edges, Rate.mat, root.state, simmap.tree=FALSE, scaleHeight=FALSE){
+varcov.ou <- function(phy, edges, Rate.mat, root.state, simmap.tree=FALSE, root.age=NULL, scaleHeight=FALSE){
     if(is.null(root.state)) {
         root.state<-which(edges[dim(edges)[1],]==1)-5
         edges<-edges[-1*dim(edges)[1],]
@@ -31,7 +31,7 @@ varcov.ou <- function(phy, edges, Rate.mat, root.state, simmap.tree=FALSE, scale
             desc = edges[i, 3]
             
             if(scaleHeight==TRUE){
-                currentmap<-phy$maps[[i]]/max(nodeHeights(phy))
+                currentmap<-phy$maps[[i]]/max(MakeAgeTable(phy, root.age=root.age))
             }
             else{
                 currentmap<-phy$maps[[i]]
@@ -104,7 +104,10 @@ varcov.ou <- function(phy, edges, Rate.mat, root.state, simmap.tree=FALSE, scale
         diag(species.total.variances) <- exp(-2 *diag(vcv1))
         vcv <- species.total.variances * vcv2
     }else{
-        vcv<-exp(-2*alpha[1]*max(branching.times(phy)))*vcv2
+        if(is.null(root.age)){
+            root.age <- max(branching.times(phy))
+        }
+        vcv<-exp(-2*alpha[1]*max(root.age))*vcv2
     }
     vcv
     
